@@ -1,10 +1,10 @@
 /* block/test_complex_source.c
  * 
- * Copyright (C) 1996, 1997, 1998, 1999, 2000 Gerard Jungman, Brian Gough
+ * Copyright (C) 1996, 1997, 1998, 1999, 2000, 2007 Gerard Jungman, Brian Gough
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 3 of the License, or (at
  * your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful, but
@@ -110,13 +110,26 @@ FUNCTION (test, binary) (void)
 
 }
 
-
 void
-FUNCTION (test, trap) (void)
+FUNCTION (test, alloc_zero_length) (void)
 {
   TYPE (gsl_block) * b = FUNCTION (gsl_block, alloc) (0);
 
-  gsl_test (b != 0, NAME (gsl_block) "_alloc traps zero length");
+  gsl_test (b == 0, NAME (gsl_block) "_alloc permits zero length");
+  gsl_test (b->size != 0, NAME (gsl_block) "_alloc reflects zero length");
+
+  FUNCTION (gsl_block, free) (b);
+}
+
+void
+FUNCTION (test, calloc_zero_length) (void)
+{
+  TYPE (gsl_block) * b = FUNCTION (gsl_block, calloc) (0);
+
+  gsl_test (b == 0, NAME (gsl_block) "_calloc permits zero length");
+  gsl_test (b->size != 0, NAME (gsl_block) "_calloc reflects zero length");
+
+  FUNCTION (gsl_block, free) (b);
 }
 
 
