@@ -9,13 +9,11 @@
 int
 main (void)
 {
-  int i;
-  double data[2*128];
+  int i; double data[2*128];
 
   for (i = 0; i < 128; i++)
     {
-       REAL(data,i) = 0.0;
-       IMAG(data,i) = 0.0;
+       REAL(data,i) = 0.0; IMAG(data,i) = 0.0;
     }
 
   REAL(data,0) = 1.0;
@@ -30,7 +28,7 @@ main (void)
       printf ("%d %e %e\n", i, 
               REAL(data,i), IMAG(data,i));
     }
-  printf ("\n");
+  printf ("\n\n");
 
   gsl_fft_complex_radix2_forward (data, 1, 128);
 
