@@ -1,6 +1,7 @@
 /* linalg/qr.c
  * 
  * Copyright (C) 1996, 1997, 1998, 1999, 2000, 2007 Gerard Jungman, Brian Gough
+ * Copyright (C) 2019 Patrick Alken
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,6 +58,50 @@
 
 int
 gsl_linalg_QR_decomp (gsl_matrix * A, gsl_vector * tau)
+{
+  const size_t N = A->size2;
+
+  if (tau->size != N)
+    {
+      return gsl_linalg_QR_decomp_old (A, tau);
+    }
+  else
+    {
+      const size_t M = A->size1;
+      size_t i;
+
+      for (i = 0; i < GSL_MIN (M, N); i++)
+        {
+          /* Compute the Householder transformation to reduce the j-th
+             column of the matrix to a multiple of the j-th unit vector */
+
+          gsl_vector_view c = gsl_matrix_subcolumn (A, i, i, M - i);
+          double tau_i = gsl_linalg_householder_transform (&(c.vector));
+          double * ptr = gsl_vector_ptr(&(c.vector), 0);
+
+          gsl_vector_set (tau, i, tau_i);
+
+          /* Apply the transformation to the remaining columns and
+             update the norms */
+
+          if (i + 1 < N)
+            {
+              gsl_matrix_view m = gsl_matrix_submatrix (A, i, i + 1, M - i, N - i - 1);
+              gsl_vector_view work = gsl_vector_subvector(tau, i + 1, N - i - 1);
+              double tmp = *ptr;
+
+              *ptr = 1.0;
+              gsl_linalg_householder_left(tau_i, &(c.vector), &(m.matrix), &(work.vector));
+              *ptr = tmp;
+            }
+        }
+
+      return GSL_SUCCESS;
+    }
+}
+
+int
+gsl_linalg_QR_decomp_old (gsl_matrix * A, gsl_vector * tau)
 {
   const size_t M = A->size1;
   const size_t N = A->size2;
@@ -74,8 +119,7 @@ gsl_linalg_QR_decomp (gsl_matrix * A, gsl_vector * tau)
           /* Compute the Householder transformation to reduce the j-th
              column of the matrix to a multiple of the j-th unit vector */
 
-          gsl_vector_view c_full = gsl_matrix_column (A, i);
-          gsl_vector_view c = gsl_vector_subvector (&(c_full.vector), i, M-i);
+          gsl_vector_view c = gsl_matrix_subcolumn (A, i, i, M - i);
 
           double tau_i = gsl_linalg_householder_transform (&(c.vector));
 
@@ -119,12 +163,10 @@ gsl_linalg_QR_solve (const gsl_matrix * QR, const gsl_vector * tau, const gsl_ve
     }
   else
     {
-      /* Copy x <- b */
-
+      /* copy x <- b */
       gsl_vector_memcpy (x, b);
 
-      /* Solve for x */
-
+      /* solve for x */
       gsl_linalg_QR_svx (QR, tau, x);
 
       return GSL_SUCCESS;
@@ -153,11 +195,9 @@ gsl_linalg_QR_svx (const gsl_matrix * QR, const gsl_vector * tau, gsl_vector * x
   else
     {
       /* compute rhs = Q^T b */
-
       gsl_linalg_QR_QTvec (QR, tau, x);
 
       /* Solve R x = rhs, storing x in-place */
-
       gsl_blas_dtrsv (CblasUpper, CblasNoTrans, CblasNonUnit, QR, x);
 
       return GSL_SUCCESS;
@@ -202,17 +242,14 @@ gsl_linalg_QR_lssolve (const gsl_matrix * QR, const gsl_vector * tau, const gsl_
       gsl_vector_memcpy(residual, b);
 
       /* compute rhs = Q^T b */
-
       gsl_linalg_QR_QTvec (QR, tau, residual);
 
       /* Solve R x = rhs */
-
       gsl_vector_memcpy(x, &(c.vector));
 
       gsl_blas_dtrsv (CblasUpper, CblasNoTrans, CblasNonUnit, &(R.matrix), x);
 
       /* Compute residual = b - A x = Q (Q^T b - R x) */
-      
       gsl_vector_set_zero(&(c.vector));
 
       gsl_linalg_QR_Qvec(QR, tau, residual);
@@ -239,12 +276,10 @@ gsl_linalg_QR_Rsolve (const gsl_matrix * QR, const gsl_vector * b, gsl_vector *
     }
   else
     {
-      /* Copy x <- b */
-
+      /* copy x <- b */
       gsl_vector_memcpy (x, b);
 
-      /* Solve R x = b, storing x in-place */
-
+      /* solve R x = b, storing x in-place */
       gsl_blas_dtrsv (CblasUpper, CblasNoTrans, CblasNonUnit, QR, x);
 
       return GSL_SUCCESS;
@@ -265,8 +300,7 @@ gsl_linalg_QR_Rsvx (const gsl_matrix * QR, gsl_vector * x)
     }
   else
     {
-      /* Solve R x = b, storing x in-place */
-
+      /* solve R x = b, storing x in-place */
       gsl_blas_dtrsv (CblasUpper, CblasNoTrans, CblasNonUnit, QR, x);
 
       return GSL_SUCCESS;
@@ -290,12 +324,10 @@ gsl_linalg_R_solve (const gsl_matrix * R, const gsl_vector * b, gsl_vector * x)
     }
   else
     {
-      /* Copy x <- b */
-
+      /* copy x <- b */
       gsl_vector_memcpy (x, b);
 
-      /* Solve R x = b, storing x inplace in b */
-
+      /* solve R x = b, storing x inplace in b */
       gsl_blas_dtrsv (CblasUpper, CblasNoTrans, CblasNonUnit, R, x);
 
       return GSL_SUCCESS;
@@ -315,8 +347,7 @@ gsl_linalg_R_svx (const gsl_matrix * R, gsl_vector * x)
     }
   else
     {
-      /* Solve R x = b, storing x inplace in b */
-
+      /* solve R x = b, storing x inplace in b */
       gsl_blas_dtrsv (CblasUpper, CblasNoTrans, CblasNonUnit, R, x);
 
       return GSL_SUCCESS;
@@ -325,8 +356,7 @@ gsl_linalg_R_svx (const gsl_matrix * R, gsl_vector * x)
 
 
 
-/* Form the product Q^T v  from a QR factorized matrix 
- */
+/* Form the product Q^T v  from a QR factorized matrix */
 
 int
 gsl_linalg_QR_QTvec (const gsl_matrix * QR, const gsl_vector * tau, gsl_vector * v)
@@ -347,15 +377,14 @@ gsl_linalg_QR_QTvec (const gsl_matrix * QR, const gsl_vector * tau, gsl_vector *
       size_t i;
 
       /* compute Q^T v */
-
       for (i = 0; i < GSL_MIN (M, N); i++)
         {
-          gsl_vector_const_view c = gsl_matrix_const_column (QR, i);
-          gsl_vector_const_view h = gsl_vector_const_subvector (&(c.vector), i, M - i);
+          gsl_vector_const_view h = gsl_matrix_const_subcolumn (QR, i, i, M - i);
           gsl_vector_view w = gsl_vector_subvector (v, i, M - i);
           double ti = gsl_vector_get (tau, i);
           gsl_linalg_householder_hv (ti, &(h.vector), &(w.vector));
         }
+
       return GSL_SUCCESS;
     }
 }
@@ -379,8 +408,7 @@ gsl_linalg_QR_Qvec (const gsl_matrix * QR, const gsl_vector * tau, gsl_vector *
     {
       size_t i;
 
-      /* compute Q^T v */
-
+      /* compute Q v */
       for (i = GSL_MIN (M, N); i-- > 0;)
         {
           gsl_vector_const_view c = gsl_matrix_const_column (QR, i);
@@ -390,11 +418,12 @@ gsl_linalg_QR_Qvec (const gsl_matrix * QR, const gsl_vector * tau, gsl_vector *
           double ti = gsl_vector_get (tau, i);
           gsl_linalg_householder_hv (ti, &h.vector, &w.vector);
         }
+
       return GSL_SUCCESS;
     }
 }
 
-/* Form the product Q^T A  from a QR factorized matrix */
+/* Form the product Q^T A from a QR factorized matrix */
 
 int
 gsl_linalg_QR_QTmat (const gsl_matrix * QR, const gsl_vector * tau, gsl_matrix * A)
@@ -415,7 +444,6 @@ gsl_linalg_QR_QTmat (const gsl_matrix * QR, const gsl_vector * tau, gsl_matrix *
       size_t i;
 
       /* compute Q^T A */
-
       for (i = 0; i < GSL_MIN (M, N); i++)
         {
           gsl_vector_const_view c = gsl_matrix_const_column (QR, i);
@@ -424,6 +452,40 @@ gsl_linalg_QR_QTmat (const gsl_matrix * QR, const gsl_vector * tau, gsl_matrix *
           double ti = gsl_vector_get (tau, i);
           gsl_linalg_householder_hm (ti, &(h.vector), &(m.matrix));
         }
+
+      return GSL_SUCCESS;
+    }
+}
+
+/* Form the product A Q from a QR factorized matrix */
+int
+gsl_linalg_QR_matQ (const gsl_matrix * QR, const gsl_vector * tau, gsl_matrix * A)
+{
+  const size_t M = QR->size1;
+  const size_t N = QR->size2;
+
+  if (tau->size != GSL_MIN (M, N))
+    {
+      GSL_ERROR ("size of tau must be MIN(M,N)", GSL_EBADLEN);
+    }
+  else if (A->size2 != M)
+    {
+      GSL_ERROR ("matrix must have M columns", GSL_EBADLEN);
+    }
+  else
+    {
+      size_t i;
+
+      /* compute A Q */
+      for (i = 0; i < GSL_MIN (M, N); i++)
+        {
+          gsl_vector_const_view c = gsl_matrix_const_column (QR, i);
+          gsl_vector_const_view h = gsl_vector_const_subvector (&(c.vector), i, M - i);
+          gsl_matrix_view m = gsl_matrix_submatrix(A, 0, i, A->size1, M - i);
+          double ti = gsl_vector_get (tau, i);
+          gsl_linalg_householder_mh (ti, &(h.vector), &(m.matrix));
+        }
+
       return GSL_SUCCESS;
     }
 }
@@ -444,30 +506,26 @@ gsl_linalg_QR_unpack (const gsl_matrix * QR, const gsl_vector * tau, gsl_matrix
     {
       GSL_ERROR ("R matrix must be M x N", GSL_ENOTSQR);
     }
-  else if (tau->size != GSL_MIN (M, N))
+  else if (tau->size < GSL_MIN (M, N))
     {
-      GSL_ERROR ("size of tau must be MIN(M,N)", GSL_EBADLEN);
+      GSL_ERROR ("size of tau must be at least MIN(M,N)", GSL_EBADLEN);
     }
   else
     {
       size_t i, j;
 
       /* Initialize Q to the identity */
-
       gsl_matrix_set_identity (Q);
 
       for (i = GSL_MIN (M, N); i-- > 0;)
         {
-          gsl_vector_const_view c = gsl_matrix_const_column (QR, i);
-          gsl_vector_const_view h = gsl_vector_const_subvector (&c.vector,
-                                                                i, M - i);
+          gsl_vector_const_view h = gsl_matrix_const_subcolumn (QR, i, i, M - i);
           gsl_matrix_view m = gsl_matrix_submatrix (Q, i, i, M - i, M - i);
           double ti = gsl_vector_get (tau, i);
           gsl_linalg_householder_hm (ti, &h.vector, &m.matrix);
         }
 
-      /*  Form the right triangular matrix R from a packed QR matrix */
-
+      /*  form the right triangular matrix R from a packed QR matrix */
       for (i = 0; i < M; i++)
         {
           for (j = 0; j < i && j < N; j++)
@@ -583,13 +641,36 @@ gsl_linalg_QR_QRsolve (gsl_matrix * Q, gsl_matrix * R, const gsl_vector * b, gsl
   else
     {
       /* compute sol = Q^T b */
-
       gsl_blas_dgemv (CblasTrans, 1.0, Q, b, 0.0, x);
 
-      /* Solve R x = sol, storing x in-place */
-
+      /* solve R x = sol, storing x in-place */
       gsl_blas_dtrsv (CblasUpper, CblasNoTrans, CblasNonUnit, R, x);
 
       return GSL_SUCCESS;
     }
 }
+
+int
+gsl_linalg_QR_rcond(const gsl_matrix * QR, double * rcond, gsl_vector * work)
+{
+  const size_t M = QR->size1;
+  const size_t N = QR->size2;
+
+  if (M < N)
+    {
+      GSL_ERROR ("M must be >= N", GSL_EBADLEN);
+    }
+  else if (work->size != 3 * N)
+    {
+      GSL_ERROR ("work vector must have length 3*N", GSL_EBADLEN);
+    }
+  else
+    {
+      gsl_matrix_const_view R = gsl_matrix_const_submatrix (QR, 0, 0, N, N);
+      int status;
+
+      status = gsl_linalg_tri_rcond(CblasUpper, &R.matrix, rcond, work);
+
+      return status;
+    }
+}
