@@ -1,10 +1,10 @@
 /* matrix/oper_source.c
  * 
- * Copyright (C) 1996, 1997, 1998, 1999, 2000 Brian Gough
+ * Copyright (C) 1996, 1997, 1998, 1999, 2000, 2007 Brian Gough
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 3 of the License, or (at
  * your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful, but
@@ -14,7 +14,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 int 
@@ -133,8 +133,8 @@ FUNCTION(gsl_matrix, div_elements) (TYPE(gsl_matrix) * a, const TYPE(gsl_matrix)
     }
 }
 
-int 
-FUNCTION(gsl_matrix, scale) (TYPE(gsl_matrix) * a, const double x)
+int
+FUNCTION(gsl_matrix, scale) (TYPE(gsl_matrix) * a, const ATOMIC x)
 {
   const size_t M = a->size1;
   const size_t N = a->size2;
@@ -153,8 +153,56 @@ FUNCTION(gsl_matrix, scale) (TYPE(gsl_matrix) * a, const double x)
   return GSL_SUCCESS;
 }
 
+int
+FUNCTION(gsl_matrix, scale_rows) (TYPE(gsl_matrix) * a, const TYPE(gsl_vector) * x)
+{
+  const size_t M = a->size1;
+
+  if (x->size != M)
+    {
+      GSL_ERROR ("x must match number of rows of A", GSL_EBADLEN);
+    }
+  else
+    {
+      size_t i;
+
+      for (i = 0; i < M; ++i)
+        {
+          const ATOMIC xi = FUNCTION (gsl_vector, get) (x, i);
+          VIEW (gsl_vector, view) v = FUNCTION (gsl_matrix, row) (a, i);
+          FUNCTION (gsl_vector, scale) (&v.vector, xi);
+        }
+
+      return GSL_SUCCESS;
+    }
+}
+
+int
+FUNCTION(gsl_matrix, scale_columns) (TYPE(gsl_matrix) * a, const TYPE(gsl_vector) * x)
+{
+  const size_t N = a->size2;
+
+  if (x->size != N)
+    {
+      GSL_ERROR ("x must match number of columns of A", GSL_EBADLEN);
+    }
+  else
+    {
+      size_t i;
+
+      for (i = 0; i < N; ++i)
+        {
+          const ATOMIC xi = FUNCTION (gsl_vector, get) (x, i);
+          VIEW (gsl_vector, view) v = FUNCTION (gsl_matrix, column) (a, i);
+          FUNCTION (gsl_vector, scale) (&v.vector, xi);
+        }
+
+      return GSL_SUCCESS;
+    }
+}
+
 int 
-FUNCTION(gsl_matrix, add_constant) (TYPE(gsl_matrix) * a, const double x)
+FUNCTION(gsl_matrix, add_constant) (TYPE(gsl_matrix) * a, const ATOMIC x)
 {
   const size_t M = a->size1;
   const size_t N = a->size2;
@@ -175,7 +223,7 @@ FUNCTION(gsl_matrix, add_constant) (TYPE(gsl_matrix) * a, const double x)
 
 
 int 
-FUNCTION(gsl_matrix, add_diagonal) (TYPE(gsl_matrix) * a, const double x)
+FUNCTION(gsl_matrix, add_diagonal) (TYPE(gsl_matrix) * a, const ATOMIC x)
 {
   const size_t M = a->size1;
   const size_t N = a->size2;
