@@ -1,10 +1,11 @@
 /* randist/exppow.c
  * 
- * Copyright (C) 1996, 1997, 1998, 1999, 2000 James Theiler, Brian Gough
+ * Copyright (C) 1996, 1997, 1998, 1999, 2000, 2006, 2007 James Theiler, Brian Gough
+ * Copyright (C) 2006 Giulio Bottazzi
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 3 of the License, or (at
  * your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful, but
@@ -37,7 +38,7 @@
 
    We use this relation for b < 1. For b >=1 we use rejection methods
    based on the laplace and gaussian distributions which should be
-   faster.
+   faster.  For b>4 we revert to the gamma method.
 
    See P. R. Tadikamalla, "Random Sampling from the Exponential Power
    Distribution", Journal of the American Statistical Association,
@@ -48,85 +49,74 @@
 double
 gsl_ran_exppow (const gsl_rng * r, const double a, const double b)
 {
-  if (b < 1) 
+  if (b < 1 || b > 4)
     {
-      double u = gsl_rng_uniform (r) ;
-      double v = gsl_ran_gamma (r, 1/b, 1.0) ;
-      double z = a * pow(v, 1/b) ;
+      double u = gsl_rng_uniform (r);
+      double v = gsl_ran_gamma (r, 1 / b, 1.0);
+      double z = a * pow (v, 1 / b);
 
-      if (u > 0.5) 
+      if (u > 0.5)
         {
-          return z ;
-        } 
-      else 
+          return z;
+        }
+      else
         {
-          return -z ;
+          return -z;
         }
     }
-  else if (b == 1) 
+  else if (b == 1)
     {
       /* Laplace distribution */
-      return gsl_ran_laplace (r, a) ;
+      return gsl_ran_laplace (r, a);
     }
-  else if (b < 2) 
+  else if (b < 2)
     {
-      /* Use laplace distribution for rejection method */
+      /* Use laplace distribution for rejection method, from Tadikamalla */
 
-      double x, y, h, ratio, u ;
+      double x, h, u;
 
-      /* Scale factor chosen by upper bound on ratio at b = 2 */
+      double B = pow (1 / b, 1 / b);
 
-      double s = 1.4489 ; 
-      do 
+      do
         {
-          x = gsl_ran_laplace (r, a) ;
-          y = gsl_ran_laplace_pdf (x,a) ;
-          h = gsl_ran_exppow_pdf (x,a,b) ;
-          ratio = h/(s * y) ;
-          u = gsl_rng_uniform (r) ;
-        } 
-      while (u > ratio) ;
-      
-      return x ;
+          x = gsl_ran_laplace (r, B);
+          u = gsl_rng_uniform_pos (r);
+          h = -pow (fabs (x), b) + fabs (x) / B - 1 + (1 / b);
+        }
+      while (log (u) > h);
+
+      return a * x;
     }
   else if (b == 2)
     {
       /* Gaussian distribution */
-      return gsl_ran_gaussian (r, a/sqrt(2.0)) ;
+      return gsl_ran_gaussian (r, a / sqrt (2.0));
     }
   else
     {
-      /* Use gaussian for rejection method */
+      /* Use gaussian for rejection method, from Tadikamalla */
 
-      double x, y, h, ratio, u ;
-      const double sigma = a / sqrt(2.0) ;
+      double x, h, u;
 
-      /* Scale factor chosen by upper bound on ratio at b = infinity.
-         This could be improved by using a rational function
-         approximation to the bounding curve. */
+      double B = pow (1 / b, 1 / b);
 
-      double s = 2.4091 ;  /* this is sqrt(pi) e / 2 */
-
-      do 
+      do
         {
-          x = gsl_ran_gaussian (r, sigma) ;
-          y = gsl_ran_gaussian_pdf (x, sigma) ;
-          h = gsl_ran_exppow_pdf (x, a, b) ;
-          ratio = h/(s * y) ;
-          u = gsl_rng_uniform (r) ;
-        } 
-      while (u > ratio) ;
+          x = gsl_ran_gaussian (r, B);
+          u = gsl_rng_uniform_pos (r);
+          h = -pow (fabs (x), b) + (x * x) / (2 * B * B) + (1 / b) - 0.5;
+        }
+      while (log (u) > h);
 
-      return x;
+      return a * x;
     }
 }
 
 double
 gsl_ran_exppow_pdf (const double x, const double a, const double b)
 {
-  double p ;
-  double lngamma = gsl_sf_lngamma (1+1/b) ;
-  p = (1/(2*a)) * exp(-pow(fabs(x/a),b) - lngamma);
+  double p;
+  double lngamma = gsl_sf_lngamma (1 + 1 / b);
+  p = (1 / (2 * a)) * exp (-pow (fabs (x / a), b) - lngamma);
   return p;
 }
-
